@@ -1,9 +1,9 @@
 package DateTime::Format::Alami;
 
-our $DATE = '2016-06-29'; # DATE
-our $VERSION = '0.11'; # VERSION
+our $DATE = '2016-06-30'; # DATE
+our $VERSION = '0.12'; # VERSION
 
-use 5.010001;
+use 5.014000;
 use strict;
 use warnings;
 use Log::Any::IfLOG '$log';
@@ -11,6 +11,7 @@ use Log::Any::IfLOG '$log';
 use Role::Tiny;
 
 my @short_mons = qw(jan feb mar apr may jun jul aug sep oct nov dec);
+my @dow = qw(monday tuesday wednesday thursday friday saturday sunday);
 
 requires 'o_num';
 requires '_parse_num';
@@ -24,6 +25,7 @@ requires 'w_minute';
 requires 'w_second';
 
 requires "w_$_" for @short_mons;
+requires "w_$_" for @dow;
 
 requires 'p_now';
 requires 'p_today';
@@ -33,6 +35,7 @@ requires 'p_dateymd';
 requires 'o_date';
 requires 'p_dur_ago';
 requires 'p_dur_later';
+requires 'p_which_dow';
 requires 'p_time';
 requires 'p_date_time';
 
@@ -126,8 +129,23 @@ sub new {
                 }
             }
         }
+        # day-of-week names -> num (monday=1, sunday=7)
+        {
+            my $i = 0;
+            for my $m (@dow) {
+                ++$i;
+                my $meth = "w_$m";
+                for (@{ $self->$meth }) {
+                    $maps->{dow}{$_} = $i;
+                }
+            }
+        }
         ${"$class\::MAPS"} = $maps;
     }
+
+    # _time_zone is old name (<= 0.11) will be removed later
+    $self->{time_zone} //= $self->{_time_zone};
+
     $self;
 }
 
@@ -139,31 +157,41 @@ sub _reset {
 }
 
 sub parse_datetime {
-    require DateTime;
     no strict 'refs';
 
+    # we require DateTime here, for all the a_* methods
+    require DateTime;
+
     my ($self, $str, $opts) = @_;
 
+    # allow calling as static method
+    unless (ref $self) { $self = $self->new }
+
     $opts //= {};
     $opts->{format} //= 'DateTime';
     #$opts->{prefers} //= 'nearest';
     $opts->{returns} //= 'first';
 
-    local $self->{_time_zone} = $opts->{time_zone} if $opts->{time_zone};
+    local $self->{time_zone} = $opts->{time_zone} if $opts->{time_zone};
 
-    my $re = ${ref($self).'::RE_DT'};
+    # we need /o to avoid repeated regcomp, but we need to make it work with all
+    # subclasses, so we use eval() here.
+    unless (defined *{ref($self).'::_code_match_dt'}) {
+        *{ref($self).'::_code_match_dt'} = eval "sub { \$_[0] =~ /(\$".ref($self)."::RE_DT)/go; \$1 }";
+        die if $@;
+    }
 
     $o = $self;
     my @res;
     while (1) {
         $o->_reset;
         $m = {};
-        $str =~ /($re)/go or last;
+        my $match = &{ref($self).'::_code_match_dt'}($str) or last;
         $o->{_dt}->truncate(to=>'day') unless $o->{_uses_time};
         my $res = {
-            verbatim => $1,
+            verbatim => $match,
             pattern => $o->{_pat},
-            pos => pos($str) - length($1),
+            pos => pos($str) - length($match),
             m => {%$m},
         };
         $res->{uses_time} = $o->{_uses_time} ? 1:0;
@@ -174,7 +202,7 @@ sub parse_datetime {
         last if $opts->{returns} eq 'first';
     }
 
-    return undef unless @res;
+    die "Can't parse date '$str'" unless @res;
 
     @res = ($res[-1]) if $opts->{returns} eq 'last';
 
@@ -212,26 +240,38 @@ sub _reset_dur {
 }
 
 sub parse_datetime_duration {
+    # we require DateTime here, for all the adur_* methods
+    require DateTime;
+    require DateTime::Duration;
+
     no strict 'refs';
 
     my ($self, $str, $opts) = @_;
 
+    # allow calling as static method
+    unless (ref $self) { $self = $self->new }
+
     $opts //= {};
     $opts->{format}  //= 'Duration';
     $opts->{returns} //= 'first';
 
-    my $re = ${ref($self).'::RE_DUR'};
+    # we need /o to avoid repeated regcomp, but we need to make it work with all
+    # subclasses, so we use eval() here.
+    unless (defined *{ref($self).'::_code_match_dur'}) {
+        *{ref($self).'::_code_match_dur'} = eval "sub { \$_[0] =~ /(\$".ref($self)."::RE_DUR)/go; \$1 }";
+        die if $@;
+    }
 
     $o = $self;
     my @res;
     while (1) {
         $o->_reset_dur;
         $m = {};
-        $str =~ /($re)/go or last;
+        my $match = &{ref($self).'::_code_match_dur'}($str) or last;
         my $res = {
-            verbatim => $1,
+            verbatim => $match,
             pattern => $o->{_pat},
-            pos => pos($str) - length($1),
+            pos => pos($str) - length($match),
             m => {%$m},
         };
         $res->{Duration}  = $o->{_dtdur};
@@ -251,14 +291,12 @@ sub parse_datetime_duration {
         last if $opts->{returns} eq 'first';
     }
 
-    return undef unless @res;
+    die "Can't parse duration" unless @res;
 
     @res = ($res[-1]) if $opts->{returns} eq 'last';
 
     # XXX support returns largest, smallest, all_sorted
     if ($opts->{returns} =~ /\A(?:all_sorted|largest|smallest)\z/) {
-        require DateTime;
-        require DateTime::Duration;
         my $base_dt = DateTime->now;
         # sort from smallest to largest
         @res = sort {
@@ -305,6 +343,14 @@ sub o_monthname {
     ) . ")";
 }
 
+sub o_dow {
+    my $self = shift;
+    "(?:" . join(
+        "|",
+        (map {my $meth="w_$_"; @{ $self->$meth }} @dow)
+    ) . ")";
+}
+
 sub o_durwords  {
     my $self = shift;
     "(?:" . join(
@@ -387,7 +433,6 @@ sub _parse_dur {
             $args{years} = $n;
         }
     }
-    require DateTime::Duration;
     DateTime::Duration->new(%args);
 }
 
@@ -404,7 +449,7 @@ sub _today_if_unset {
 sub a_now {
     my $self = shift;
     $self->{_dt} = DateTime->now(
-        (time_zone => $self->{_time_zone}) x !!defined($self->{_time_zone}),
+        (time_zone => $self->{time_zone}) x !!defined($self->{time_zone}),
     );
     $self->{_uses_time} = 1;
 }
@@ -412,7 +457,7 @@ sub a_now {
 sub a_today {
     my $self = shift;
     $self->{_dt} = DateTime->today(
-        (time_zone => $self->{_time_zone}) x !!defined($self->{_time_zone}),
+        (time_zone => $self->{time_zone}) x !!defined($self->{time_zone}),
     );
     $self->{_uses_time} = 0;
 }
@@ -455,6 +500,23 @@ sub a_dateymd {
     }
 }
 
+sub a_which_dow {
+    no strict 'refs';
+
+    my ($self, $m) = @_;
+    $self->a_today;
+    my $dow_num = $self->{_dt}->day_of_week;
+
+    my $maps = ${ ref($self) . '::MAPS' };
+    my $wanted_dow_num = $maps->{dow}{lc $m->{o_dow} };
+
+    $self->{_dt}->add(days => ($wanted_dow_num-$dow_num));
+
+    if ($m->{offset}) {
+        $self->{_dt}->add(days => (7*$m->{offset}));
+    }
+}
+
 sub adur_dur {
     my ($self, $m) = @_;
     $self->{_dtdur} = $self->_parse_dur($m->{odur_dur});
@@ -507,31 +569,44 @@ DateTime::Format::Alami - Parse human date/time expression (base class)
 
 =head1 VERSION
 
-This document describes version 0.11 of DateTime::Format::Alami (from Perl distribution DateTime-Format-Alami), released on 2016-06-29.
+This document describes version 0.12 of DateTime::Format::Alami (from Perl distribution DateTime-Format-Alami), released on 2016-06-30.
 
 =head1 SYNOPSIS
 
-Use English:
+For English:
 
  use DateTime::Format::Alami::EN;
  my $parser = DateTime::Format::Alami::EN->new();
- my $dt;
- $dt = $parser->parse_datetime("2 hours 13 minutes from now");
- $dt = $parser->parse_datetime("yesterday");
+ my $dt = $parser->parse_datetime("2 hours 13 minutes from now");
 
-use Indonesian:
+Or you can also call as class method:
+
+ my $dt = DateTime::Format::Alami::EN->parse_datetime("yesterday");
+
+To parse duration:
+
+ my $dtdur = DateTime::Format::Alami::EN->parse_datetime_duration("2h"); # 2 hours
+
+For Indonesian:
 
  use DateTime::Format::Alami::ID;
  my $parser = DateTime::Format::Alami::ID->new();
- my $dt;
- $dt = $parser->parse_datetime("5 jam lagi");
- $dt = $parser->parse_datetime("hari ini");
+ my $dt = $parser->parse_datetime("5 jam lagi");
+
+Or you can also call as class method:
+
+ my $dt = DateTime::Format::Alami::ID->parse_datetime("hari ini");
+
+To parse duration:
+
+ my $dtdur = DateTime::Format::Alami::ID->parse_datetime_duration("2h"); # 2 days
 
 =head1 DESCRIPTION
 
-This class parses human/natural date/time string and returns DateTime object.
-Currently it supports English and Indonesian. The goal of this module is to make
-it easier to add support for other human languages.
+This class parses human/natural date/time/duration string and returns
+L<DateTime> (or L<DateTime::Duration>) object. Currently it supports English and
+Indonesian. The goal of this module is to make it easier to add support for
+other human languages.
 
 To actually use this class, you must use one of its subclasses for each
 human language that you want to parse.
@@ -613,8 +688,8 @@ Constructor. You actually must instantiate subclass instead.
 
 =head2 parse_datetime($str[ , \%opts ]) => obj
 
-Parse/extract date/time expression in C<$str>. Return undef if expression cannot
-be parsed. Otherwise return L<DateTime> object (or string/number if C<format>
+Parse/extract date/time expression in C<$str>. Die if expression cannot be
+parsed. Otherwise return L<DateTime> object (or string/number if C<format>
 option is C<verbatim>/C<epoch>, or hash if C<format> option is C<combined>) or
 array of objects/strings/numbers (if C<returns> option is C<all>/C<all_cron>).
 
@@ -668,8 +743,8 @@ instead of a single result, even if there is only a single actual result.
 
 =head2 parse_datetime_duration($str[ , \%opts ]) => obj
 
-Parse/extract duration expression in C<$str>. Return undef if expression cannot
-be parsed. Otherwise return L<DateTime::Duration> object (or string/number if
+Parse/extract duration expression in C<$str>. Die if expression cannot be
+parsed. Otherwise return L<DateTime::Duration> object (or string/number if
 C<format> option is C<verbatim>/C<seconds>, or hash if C<format> option is
 C<combined>) or array of objects/strings/numbers (if C<returns> option is
 C<all>/C<all_sorted>).
@@ -711,17 +786,46 @@ result.
 
 =head1 FAQ
 
-=head2 How does it compare to DateTime::Format::Natural?
-
-DateTime::Format::Alami::EN (DFA:EN) currently understands less English
-date/time strings than DateTime::Format::Natural (DF:Natural). DFA:EN can parse
-duration strings like "2 days" while C<parse_datetime_duration()> in DF:Natural
-tries to find one or two dates instead of duration.
-
 =head2 What does "alami" mean?
 
 It is an Indonesian word, meaning "natural".
 
+=head2 How does it compare to similar modules?
+
+L<DateTime::Format::Natural> (DF:Natural) is a more established module (first
+released on 2006) and can understand a bit more English expression like 'last
+day of Sep'. Aside from English, it does not yet support other languages.
+
+DFA:EN's C<parse_datetime_duration()> produces a L<DateTime::Duration> object
+while DF:Natural's C<parse_datetime_duration()> returns two L<DateTime> objects
+instead. In other words, DF:Natural can parse "from 23 Jun to 29 Jun" in
+addition to "for 2 weeks".
+
+DF:Natural in general is slightly more strict about the formats it accepts, e.g.
+it rejects C<Jun 23st> (the error message even gives hints that the suffix must
+be 'rd'). DF:Natural can give a detailed error message on why parsing has failed
+(see its C<error()> method).
+
+L<DateTime::Format::Flexible> (DF:Flexible) is another established module (first
+released in 2007) that, aside from parsing human expression (like 'tomorrow',
+'sep 1st') can also parse date/time in several other formats like RFC 822,
+making it a convenient module to use as a 'one-stop' solution to parse date.
+Compared to DF:Natural, it has better support for timezone but cannot parse some
+English expressions. Aside from English, it currently supports German and
+Spanish. It does not support parsing duration expression.
+
+This module itself: B<DateTime::Format::Alami> (DF:Alami) is yet another
+implementation. Internally, it uses recursive regex to make parsing simpler and
+adding more languages easier. It requires perl 5.14.0 or newer due to the use of
+C<(?{ ... })> code blocks inside regular expression (while DF:Natural and
+DF:Flexible can run on perl 5.8+). It currently supports English and Indonesian.
+It supports parsing duration expression and returns DateTime::Duration object.
+It has the smallest startup time (see see
+L<Bencher::Scenario::DateTimeFormatAlami::Startup>).
+
+Performance-wise, all the modules are within the same order of magnitude (see
+L<Bencher::Scenario::DateTimeFormatAlami::Parsing>).
+
 =head1 HOMEPAGE
 
 Please visit the project's homepage at L<https://metacpan.org/release/DateTime-Format-Alami>.
@@ -745,11 +849,12 @@ feature.
 L<Date::Extract>. DateTime::Format::Alami has some features of Date::Extract so
 it can be used to replace Date::Extract.
 
-For Indonesian: L<DateTime::Format::Indonesian>, L<Date::Extract::ID> (currently
-this module uses DateTime::Format::Alami as its backend).
+L<DateTime::Format::Flexible>. See L</"FAQ">.
 
-For English: L<DateTime::Format::Natural>. You probably want to use this
-instead, unless you want something other than English.
+For Indonesian: L<DateTime::Format::Indonesian>, L<Date::Extract::ID> (currently
+this module uses DateTime::Format::Alami::ID as its backend).
+
+For English: L<DateTime::Format::Natural>. See L</"FAQ">.
 
 =head2 Other modules on CPAN
 
